/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Id: imaging_detector.h,v 1.5 2002/05/30 11:22:25 jaffe Exp $ */

#ifndef _IMAGING_DETECTOR_H
#define _IMAGING_DETECTOR_H
/*----------------------------------------------------------------------- */
/* Imaging data FITS binary tables routines                               */
/* Descriptor table for raw data table for optical/IR interferometer      */
/*----------------------------------------------------------------------- */
#include "VLTItable.h"
#include <math.h>

typedef struct      /* table definition for imaging detector table */
{
  table_def *def;      /* basic table information */
/* column numbers */
  int    region_col;      /* region column number (0-rel) */
  int    detector_col;    /* detector column  number (0-rel) */
  int    ports_col;       /* ports column  number (0-rel) */
  int    correlation_col; /* correlation column  number (0-rel) */
  int    regname_col;     /* regname column  number (0-rel) */
  int    corner_col;      /* corner column  number (0-rel) */
  int    gain_col;        /* gain column  number (0-rel) */
  int    naxis_col;       /* naxis column  number (0-rel) */
  int    crval_col;       /* crval column  number (0-rel) */
  int    crpix_col;       /* crpix column  number (0-rel) */
  int    ctype_col;       /* ctype column  number (0-rel) */
  int    cd_col;          /* cd matrix column  number (0-rel) */
  int    dmp_col;         /* dmp column  number (0-rel) */
  int    dmc_col;         /* dmc column  number (0-rel) */
  /* keyword data */
  int    ndetect;         /* number of detectors */
  int    nregion;         /* number of regions */
  int    maxtel;          /* max. number of ports contributing to a region */
  char   *instrument;     /* name of the instrument */
  char   *date_obs;       /* observing date */
  int    num_dim;         /* number of dimensions in data array */
  int    max_coef;        /* maximum number of entries in dmp, dmc arrays */
  /* table row data - one per region */
  int    *region;         /* region numbers */
  int    *detector;       /* detector numbers */
  int    **ports;         /* input port numbers contributing to region */
  char   **regname;       /* Names of the regions */
  int    *correlation;    /* correlation type */
  int    **corner;        /* corner pixel on the detector, 2 values: */
                          /* lo x, lo y */
  double *gain;           /* detector electron to ADU factor */
  int    **naxis;         /* First and second axis dimensions */
  int    ****dmp;         /* dedistortion polynomial powers */
  double **crval;         /* x and y coordinate values at reference pixel */
  float  **crpix;         /* x and y reference pixel numbers (1-rel) */
  char   ***ctype;        /* x and y axis coordinate types */
  double **cd;            /* 2x2 CD matrix */
  double ***dmc;          /* dedistortion polynomial coefficients */
} imaging_detector_table_def;

/* public functions */

imaging_detector_table_def* 
make_imaging_detector_table_def(int nregion, int ndetect, int maxtel, int num_dim,
				int max_coef,
				char *instrument, char *date_obs,
				int *region, char **regname, int *detector, 
				int **ports, int *correlation, 
				int **corner, double *gain,
				int **naxis,
				double **crval, float **crpix, char ***ctype,
				double **cd, double ***dmc, int ****dmp);
/*----------------------------------------------------------------------- */
/*   Make a new imaging_detector_table_def.                               */
/*   Inputs:                                                              */
/*      nregion  the number of regions (rows) in the table                */
/*      ndetect  the number of detectors used                             */
/*      maxtel   the max number of ports contributing to a region         */
/*      num_dim  number of dimensions in data array                       */
/*      max_coef maximum number of entries in dmp, dmc arrays             */
/*      instrument  the name of the instrument                            */
/*      date_obs    Date (yyyy-mm-dd) observations began                  */
/*      region      array of region numbers                               */
/*      regname     array of region names, NULL=>empty string             */
/*      detector    array of detector numbers, NULL => default (1)        */
/*      ports       array of sets of ports in region, NULL=default(2)     */
/*      correlation array of correlation types for regions                */
/*                  0 =background, 1=autocorrelation, 2=crosscorrelation  */
/*      corner      corner for each region.                               */
/*                  as, low x, low y, NULL => default (-1,-1).            */
/*      gain        array detector gain factors                           */
/*      naxis       array of x and y dimensions of the regions            */
/*      crval       array of x and y coordinate values at reference pixels*/
/*                  NULL => default (0)                                   */
/*      crpix       array of x and y reference pixel numbers              */
/*                  NULL => default (1)                                   */
/*      ctype       array of x an y coordinate type strings               */
/*                  NULL => default ("X", "Y")                            */
/*      cd          array of cd (transformation) matrices as              */
/*                  CD_1_1, CD_1_2, CD_2_1, CD_2_2                        */
/*                  NULL => default (1,0,0,1)                             */
/*      dmc         dedistortion polynomial coefficients[reg][term][dim]  */
/*      dmp         dedistortion polynomial powers [reg][term][dim][dim]  */
/*   Returns:                                                             */
/*      An imaging_detector_table_def pointer; NULL indicates failure     */
/*----------------------------------------------------------------------- */

imaging_detector_table_def* 
make_testing_detector_table_def(int nx, int ny, int nreg);
/*----------------------------------------------------------------------- */
/* create a, mostly dummy imaging detector table suitable for algorithm   */
/*  testing;  makes one cross correlation array nx x ny pixels            */
/*   Inputs:                                                              */
/*      nx     number of pixel in "x" direction                           */
/*      ny     number of pixel in "y" direction                           */
/*      nreg   Number of regions                                          */
/*   Returns:                                                             */
/*      An imaging_detector_table_def pointer; NULL indicates failure     */
/*----------------------------------------------------------------------- */


imaging_detector_table_def* 
read_imaging_detector_table(fitsfile *fptr, int *ierr);
/*----------------------------------------------------------------------- */
/*   Read an (entire) imaging_detector_table from an active FITS file     */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to active FITS file                     */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      An imaging_detector_table_def pointer; NULL indicates failure     */
/*----------------------------------------------------------------------- */

void write_imaging_detector_table(fitsfile *fptr, imaging_detector_table_def* table, 
			     int *ierr);
/*----------------------------------------------------------------------- */
/*   Write an (entire) imaging_detector_table to an active FITS file      */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to active FITS file                     */
/*      table     imaging_detector_table structure to write               */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*----------------------------------------------------------------------- */

void print_imaging_detector_table(imaging_detector_table_def* table);
/*----------------------------------------------------------------------- */
/*   Print an imaging_detector_table to stdout                            */
/*   Inputs:                                                              */
/*      table  pointer to structure to be printed                         */
/*----------------------------------------------------------------------- */

void kill_imaging_detector_table_def (imaging_detector_table_def* table);
/*----------------------------------------------------------------------- */
/*   Destroy an imaging_detector_table_def                                */
/*   Inputs:                                                              */
/*      table  pointer to structure to be deleted                         */
/*----------------------------------------------------------------------- */

/* */

float * imaging_detector_wcs(imaging_detector_table_def *def, int region);
/*----------------------------------------------------------------------- */
/*   Return the world coordinates for all the pixels in one region        */
/*   Inputs:                                                              */
/*      det     pointer to detector def table                             */
/*      region  which region you want information                         */
/*   Outputs:                                                             */
/*      coord      float[2][ny][nx] 1st and 2nd wcs at each point         */
/*                 for spectral obs these are freq and slit pos resp.     */
/*----------------------------------------------------------------------- */
#endif /* end IMAGING_DETECTOR_H */
